@@ -30,7 +30,7 @@ use crate::{Atom, Axis, Exact, Vertex, VertexSet, ANOMALOUS_CONFIGURATIONS, SYMM
 /// *Parallel faces share the same layout with respect to their other axes.* For example, a face on
 /// the `-Z` axis will have all of its vertices parallel w/ a face on the `Z` axis; the vertices are
 /// stored in the same order.
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct Face {
     bits: BitArray<Lsb0, [u8; 1]>,
 }
@@ -81,8 +81,19 @@ impl Face {
             }
         }
 
-        let axis_bits = axis.to_bits();
-        bits[4..7].copy_from_bitslice(&axis_bits[0..3]);
+        bits[4..7].copy_from_bitslice(&axis.to_bits()[0..3]);
+
+        Self { bits }
+    }
+
+    /// Construct an empty face on a given axis.
+    ///
+    /// Atoms can't be empty, so this is useful if you need to just conjure an empty face out of
+    /// nowhere.
+    #[inline]
+    pub fn empty(axis: Axis) -> Self {
+        let mut bits = BitArray::zeroed();
+        bits[4..7].copy_from_bitslice(&axis.to_bits()[0..3]);
 
         Self { bits }
     }
@@ -256,9 +267,9 @@ impl Face {
 /// (see [`ANOMALOUS_CONFIGURATIONS`].) These are simple to deal with as well, as they can never be
 /// collapsed into a quad, at least not within a single `Atom`, because these are the groups of
 /// three vertices which are alone on their plane within the cube.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Hash)]
 pub struct InteriorHull {
-    facets: ArrayVec<HullFacet, 8>,
+    facets: ArrayVec<HullFacet, 4>,
 }
 
 impl InteriorHull {
@@ -343,7 +354,7 @@ fn generate_interior_hull(atom: VertexSet) -> InteriorHull {
 /// and which may be clipped during `join` operations.
 ///
 /// Internally, this is a set of six [`Face`]s.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct ExteriorHull {
     faces: ArrayVec<Face, 6>,
 }
@@ -375,6 +386,18 @@ impl ExteriorHull {
     pub fn facets(&self) -> impl Iterator<Item = HullFacet> + '_ {
         self.faces.iter().copied().flat_map(Face::facets)
     }
+
+    /// Directly set a face of the exterior hull.
+    #[inline]
+    pub fn set_face(&mut self, face: Face) {
+        self.faces[face.axis().to_index()] = face;
+    }
+
+    /// Get a face of the exterior hull.
+    #[inline]
+    pub fn face(&self, axis: Axis) -> &Face {
+        &self.faces[axis.to_index()]
+    }
 }
 
 /// The "compound hull" of an atom is its convex hull, possibly with pieces missing from its
@@ -389,7 +412,7 @@ impl ExteriorHull {
 ///
 /// In the end, the interior hull of an atom should never change; however, the exterior hull may
 /// change depending on adjacent atoms.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Hash)]
 pub struct CompoundHull {
     interior: InteriorHull,
     exterior: ExteriorHull,
@@ -417,6 +440,12 @@ impl CompoundHull {
         &self.exterior
     }
 
+    /// Get the exterior component of the hull, mutably.
+    #[inline]
+    pub fn exterior_mut(&mut self) -> &mut ExteriorHull {
+        &mut self.exterior
+    }
+
     /// Join the exterior hulls of two compound hulls.
     ///
     /// See [`ExteriorHull::join`].
@@ -528,7 +557,6 @@ mod tests {
     use super::*;
     use crate::{vertex_set, FACES};
     use maplit::hashset;
-    use nalgebra::*;
     use std::collections::HashSet;
 
     fn hull(vertex_set: VertexSet) -> CompoundHull {
